// fluid_project/CDrawBox.h

#ifndef CDRAWBOX_H_INCLUDED
#define CDRAWBOX_H_INCLUDED

#include <cassert>
#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <memory> // for unique_ptr
#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/fl_draw.H>
#include <FL/names.h>

enum {
    BTN_RADIO_ID_LINE,
    BTN_RADIO_ID_CIRCLE,
    BTN_RADIO_ID_RECTANGLE
};

class CPrimitiveBase {
protected:
    int x1;
    int y1;
    int x2;
    int y2;
    Fl_Color color;
public:
    CPrimitiveBase( int x1, int y1, int x2, int y2, Fl_Color color );

    virtual void draw() = 0;
    virtual void update_x2y2( int x_coord, int y_coord ) = 0;
    virtual std::string get_type() const = 0;
    virtual ~CPrimitiveBase() {} // virtual destructor

    // operations
    std::string to_string() const;

}; // class CPrimitiveBase

// Overloaded operators
std::ostream& operator<<( std::ostream& ostr, CPrimitiveBase const& primitive );


class CPrimitiveLine : public CPrimitiveBase {
public:
    CPrimitiveLine( int x1, int y1, int x2, int y2, Fl_Color color );

    void draw();
    void update_x2y2( int x_coord, int y_coord );
    std::string get_type() const
    {
        return "L";
    }
};//class CPrimitiveLine

class CPrimitiveCircle : public CPrimitiveBase {
public:
    CPrimitiveCircle( int x1, int y1, int x2, int y2, Fl_Color color );

    void draw();
    void update_x2y2( int x_coord, int y_coord );
    std::string get_type() const
    {
        return "C";
    }

};//class CPrimitiveCircle

class CPrimitiveRectangle : public CPrimitiveBase {
public:
    CPrimitiveRectangle( int x1, int y1, int x2, int y2, Fl_Color color );

    void draw();
    void update_x2y2( int x_coord, int y_coord );
    std::string get_type() const
    {
        return "R";
    }

};//class CPrimitiveRectangle

// function decalrations
CPrimitiveBase* create_primitive(
    int type, // BTN_RADIO_ID_LINE, BTN_RADIO_ID_CIRCLE, BTN_RADIO_ID_RECTANGLE, etc
    int x1,
    int y1,
    int x2,
    int y2,
    Fl_Color color
    );

class CDrawBox : public Fl_Box {
    Fl_Color border_color;
    int push_x_coord;
    int push_y_coord;
    std::vector< std::unique_ptr< CPrimitiveBase > > vect_primitives;
    static const int DRAW_ILLEGAL_COORDINATE = -1;
    int primitive_type;

public:
    CDrawBox( int X, int Y, int W, int H )
        :
        Fl_Box( X, Y, W, H ),
        border_color( FL_BLUE ),
        push_x_coord( DRAW_ILLEGAL_COORDINATE ),
        push_y_coord( DRAW_ILLEGAL_COORDINATE ),
        primitive_type( BTN_RADIO_ID_LINE )
    {
    }

    void draw();
    void draw_primitives();
    void draw_border();
    void draw_clear();
    void use_primitive_type( int type );
    int handle( int event_ );
    void save_to_file( std::string const& filepath );
    void load_from_file( std::string const& filepath );

};// class CDrawBox

#endif // CDRAWBOX_H_INCLUDED
