// Lab 21: Scratch Pad with load and save functionality
// CMainWindow.cpp

#include "CMainWindow.h"

void CMainWindow::show()
{
    // Window callbacks:
    win_app->callback( (Fl_Callback*)CMainWindow::callback_window_closing, (void*)(this) );
    rdo_line->callback( (Fl_Callback*)CMainWindow::callback_radio_button, (void*)( new CRadioSelector(this, BTN_RADIO_ID_LINE ) ) );
    rdo_circle->callback( (Fl_Callback*)CMainWindow::callback_radio_button, (void*)( new CRadioSelector(this, BTN_RADIO_ID_CIRCLE ) ) );
    rdo_rectangle->callback( (Fl_Callback*)CMainWindow::callback_radio_button, (void*)( new CRadioSelector(this, BTN_RADIO_ID_RECTANGLE ) ) );
    chk_filled->callback( (Fl_Callback*)CMainWindow::callback_check_button, (void*)(this) );

    menu_save->callback( (Fl_Callback*)CMainWindow::callback_menu_save, (void*)(this) );
    menu_load->callback( (Fl_Callback*)CMainWindow::callback_menu_load, (void*)(this) );
    menu_quit->callback( (Fl_Callback*)CMainWindow::callback_window_closing, (void*)(this) );

    rdo_line->value( 1 ); // make sure line primitive is selected by default
    chk_filled->deactivate();
    draw_box->use_primitive_type( BTN_RADIO_ID_LINE );

    // Make the window visible:
    win_app->show();
}

// callback functions
void CMainWindow::callback_window_closing(Fl_Widget* widg, void* userdata_) {
    std::cout << "X button clicked -- exiting the program\n";
    while( Fl::first_window() ) {
        Fl::first_window()->hide();
    }
}

void CMainWindow::click_radio_button( int button_id )
{
    std::cout << "radio click " << button_id << "\n";
    draw_box->use_primitive_type( button_id );
    switch ( button_id ) {
    case BTN_RADIO_ID_LINE:
        chk_filled->deactivate();
        break;

    case BTN_RADIO_ID_CIRCLE:
        chk_filled->activate();
        break;

    case BTN_RADIO_ID_RECTANGLE:
        chk_filled->activate();
        break;
    default:
        assert( !"BAD button id in CMainWindow::click_radio_button()" );
    }

}

void CMainWindow::callback_radio_button(Fl_Widget* widg, void* userdata_ )
{
    CRadioSelector* selector = static_cast< CRadioSelector* >( userdata_ );
    selector->select();
}

CRadioSelector::CRadioSelector( CMainWindow* window, int button_id )
    :
window( window ), button_id( button_id )
{
}

void CRadioSelector::select()
{
    window->click_radio_button( button_id );
}

void CMainWindow::click_check_button()
{
    std::cout << "check box click: " << int( chk_filled->value() ) << "\n";
}

void CMainWindow::callback_check_button(Fl_Widget* widg, void* userdata_)
{
    CMainWindow* window = static_cast< CMainWindow* > ( userdata_ );
    window->click_check_button();
}

void CMainWindow::click_menu_save()
{
    std::cout << "click_menu_save()\n";
    std::string filepath = select_file( "Save File" );
    draw_box->save_to_file( filepath );
    std::cout << filepath << "saved\n";
}

void CMainWindow::click_menu_load()
{
    std::cout << "click_menu_load()\n";
    std::string filepath = select_file( "Load File" );
    draw_box->load_from_file( filepath );
    std::cout << filepath << "loaded\n";
}

void CMainWindow::callback_menu_save(Fl_Widget* widg, void* userdata_)
{
    CMainWindow* window = static_cast< CMainWindow* > ( userdata_ );
    window->click_menu_save();
}

void CMainWindow::callback_menu_load(Fl_Widget* widg, void* userdata_)
{
    CMainWindow* window = static_cast< CMainWindow* > ( userdata_ );
    window->click_menu_load();
}

std::string select_file( std::string prompt )
{
    // Note: function returns empty string if file selection has been cancelled
    // Otherwise it returns full path to the file picked by the user
    Fl_Native_File_Chooser file_chooser;
    file_chooser.directory( nullptr );
    file_chooser.title( prompt.c_str() );
    file_chooser.type( Fl_Native_File_Chooser::BROWSE_FILE );

    // Note: file_chooser.type() arguments can be:
    // Fl_Native_File_Chooser::SINGLE - allows the user to select a single, existing file.
    // Fl_Native_File_Chooser::BROWSE_MULTI_FILE - allows the user to select one or more existing files.
    // Fl_Native_File_Chooser::CREATE - allows the user to select a single, existing file or specify a new filename.
    // Fl_Native_File_Chooser:: BROWSE_DIRECTORY - allows the user to select a single, existing directory.

    // Hang on until the user makes a choice
    const char* file_path = "";
    std::cout << "----------file chooser results----------\n";
    switch( file_chooser.show() ) {
    case -1:
        std::cout << "Error in Fl_Native_File_Chooser::show()\n";
        std::cout << file_chooser.errmsg(); 
        break;
    case 1:
        std::cout << "User cancelled the file selection dialog\n";
        break;
    default:
        file_path = file_chooser.filename();
        //std::cout << "DIRECTORY: " << file_chooser.directory() << '\n';
        std::cout << " FILENAME: " << file_chooser.filename() << '\n';
        //std::cout << "    COUNT: " << file_chooser.count() << '\n';
        //if ( file_chooser.count() > 1 ) {
        //    for ( int idx = 1; idx <= file_chooser.count(); ++idx ) {
        //        std::cout << " VALUE(" << idx << "): '" << file_chooser.filename( idx ) << '\n';
        //    }
        //}
	    break;
    }
    std::cout << "----------------------------------------\n";
    return file_path;
}
